<template>

  <div class="relative block items-center justify-center  w-full h-full aspect-square overflow-hidden rounded-xl">
    <a-upload-dragger
        v-model:fileList="fileList"
        name="file"
        class="!transition !duration-200 !bg-white hover:!bg-gray-200"
        :before-upload="beforeUpload"
        :multiple="true"
        :max-count="maxFileLength"
        accept="image/*, video/*"
        :action="() => {}"
        @change="handleChange"
        :percent="handleProgress"
        @drop="handleDrop"
        @preview="handlePreview"
    >
      <div v-if="fileList.length === 0"
           class="relative flex items-center justify-center overflow-hidden h-full w-full p-4">
        <div class="relative block">
          <p class="ant-upload-drag-icon">
            <inbox-outlined :style="{fontSize: '6rem', color:'#7646FC'}"></inbox-outlined>
          </p>
          <p class="ant-upload-text">Click or drag file to this area to upload</p>
          <p class="ant-upload-hint text-italic">
            <span class="font-semibold">Supported file type:</span> .jpeg, .bmp, .png, .jpg, .mp4, .mov,
            .ogg, .qt
          </p>
        </div>
      </div>
      <div v-if="fileList.length > 0 && mediaUrl"
           class="absolute top-0 left-0 flex items-center justify-center z-10 w-full h-full aspect-square transition duration-200 cursor-pointer p-2">
        <img :src="mediaUrl"
             class="object-center max-w-full max-h-full relative block overflow-hidden "/>
      </div>
    </a-upload-dragger>
    <button v-if="fileList.length > 0" @click="removeMedia"
            class="absolute right-2 top-2 z-50 btn-sm btn btn-primary normal-case">Remove
    </button>

    <!--        <div-->
    <!--                :class="`absolute bottom-0 w-full flex justify-end z-50 p-3 bg-gradient-to-b from-transparent to-[#00000075] bg-opacity-25 transition-all duration-200 transform ${fileList.length === 0 ?  'translate-y-full invisible opacity-0' : 'translate-y-0 opacity-100 visible'}`">-->
    <!--            <button @click="onUpload"-->
    <!--                    :disabled="fileList.length === 0"-->
    <!--                    :class="`btn z-60 btn-circle btn-primary overflow-hidden p-3 flex items-center justify-center transition-all duration-200 transform ${progressPercentage > 0 && progressPercentage <= 100 ?  'translate-y-full invisible opacity-0' : 'translate-y-0'}`">-->
    <!--                <IcPaperPlane/>-->
    <!--            </button>-->
    <!--        </div>-->
    <div
        :class="`absolute top-0 left-0 w-full h-full bg-white transition-all duration-300 z-40 bg-opacity-75  ${(progressPercentage > 0 && progressPercentage <= 100) && (mediaUrl) ?  'opacity-100 visible ' : 'opacity-0 invisible'}`">
      <div class="relative flex items-center justify-center h-full w-full">
        <a-progress
            :class="`transition duration-200 transform ${progressPercentage > 0 && progressPercentage <= 100 ?  'translate-y-0 opacity-100 ' : 'translate-y-full opacity-0'}`"
            type="circle"
            :stroke-color="{
        '0%': '#6700e3',
        '100%': '#a376ff',
      }"
            :percent="progressPercentage"
        />
      </div>
    </div>
    <div
        :class="`absolute top-0 left-0 w-full h-full bg-white transition-all z-50 duration-300 bg-opacity-90 ${(mediaUrl && isProcessing) ?  'opacity-100 visible ' : 'opacity-0 invisible'}`">
      <div class="relative flex items-center justify-center h-full w-full">
        <div class="relative text-center block overflow-hidden m-auto">
          <div class="relative block overflow-hidden m-auto w-36 h-36">
            <Vue3Lottie :animation-data="loadingJson"/>
          </div>
          <h3 class="text-lg font-semibold text-scheme-primary">Processing...</h3>
        </div>
      </div>
    </div>
    <div
        v-if="mediaUrl && mediaType === 'video'"
        :class="`absolute top-0 left-0 w-full h-full bg-white transition-all duration-300 z-30 bg-opacity-90 ${(mediaUrl !== '' && mediaType === 'video') ?  'opacity-100 visible ' : 'opacity-0 invisible'}`">
      <div class="relative flex items-center justify-center h-full w-full">
        <div v-if="mediaType === 'video'"
             class="z-40 relative flex items-center justify-center overflow-hidden w-full h-full">
          <video id="idVideo" @pause="onPaused()" class="object-fill w-full h-auto">
            <source :src="mediaUrl">
            Your browser does not support the video tag.
          </video>
          <button type="button" @click="playVideo()" id="playVideo"
                  style="box-shadow: 0px 4px 4px rgba(0, 0, 0, 0.25);"
                  class="overflow-hidden w-12 h-12 bg-white transition duration-300 absolute left-1/2 top-1/2 transform -translate-y-1/2 -translate-x-1/2 hover:bg-opacity-80 p-3 rounded-full group">
            <svg class="w-full h-full" style="padding-left: 0.3em;" viewBox="0 0 22 26" fill="none"
                 xmlns="http://www.w3.org/2000/svg">
              <path
                  d="M21 11.268C22.3333 12.0378 22.3333 13.9622 21 14.7321L3 25.1244C1.66667 25.8942 3.33085e-07 24.9319 3.14725e-07 23.3923L6.68711e-08 2.60769C4.85115e-08 1.06809 1.66667 0.105845 3 0.875645L21 11.268Z"
                  fill="#7646FC"/>
            </svg>
          </button>
        </div>
      </div>
    </div>
  </div>
  <div :class="`${ ''}  relative flex justify-between  py-3 items-center w-full align-middle h-16 gap-3`">

    <div class="flex gap-4 items-center w-full h-full relative px-3">
      <a-textarea
          v-model:value="messageForm"
          @keydown.enter.exact.prevent="onKeyupMessage"
          @keydown.enter.shift.exact.prevent="onKeydownMessage"
          :bordered="false"
          class="w-full !items-center !bg-white !py-3 !rounded-lg !flex !relative  !border-0 !resize-none !leading-none px-3 !h-full !min-h-full"
          placeholder="Ketik Pesan"
          id="modal_content_media_message_form"
          auto-size
      />
      <button @click.prevent="onUpload"
              :disabled="fileList.length === 0"
              class="relative overflow-hidden flex items-center justify-center text-center h-12 bg-scheme-primary transition duration-300 hover:bg-opacity-80 p-3 rounded-full group"
      >
        <IcLoading v-if="isProcessing"/>
        <IcPaperPlane v-else/>
      </button>
      <!--                <button-->
      <!--                        @click="sendDocument"-->
      <!--                        class="right-8 absolute overflow-hidden w-12 h-12 bg-scheme-primary transition duration-300 hover:bg-opacity-80 p-3 rounded-full group"-->
      <!--                        id="btn-sendMessage">-->


      <!--                </button>-->
    </div>
  </div>
  <div v-if="fileList.length > 1"
       class="relative block mt-3">
    <div class="relative grid grid-cols-5 gap-3">

      <div v-for="(file, fileKey) in filePreview"
           class="aspect-square bg-gray-400 rounded-lg  relative">
        <button @click="removeFileList(fileKey)"
                class="btn btn-circle ant-btn-danger btn-xs absolute -top-2 -right-2 z-10">
          <svg xmlns="http://www.w3.org/2000/svg" class="fill-current text-white" viewBox="0 0 24 24">
            <path
                d="M19,6.41L17.59,5L12,10.59L6.41,5L5,6.41L10.59,12L5,17.59L6.41,19L12,13.41L17.59,19L19,17.59L13.41,12L19,6.41Z"/>
          </svg>
        </button>
        <div @click="changePreview(fileKey)"
             class="relative cursor-pointer block overflow-hidden aspect-square">
          <img :src="file.media_url"/>
        </div>
      </div>
    </div>
  </div>
</template>

<script setup lang="ts">
import {InboxOutlined} from '@ant-design/icons-vue';
import {defineComponent, ref, watch} from 'vue';
import {message} from 'ant-design-vue';
import type {UploadChangeParam, UploadProps} from 'ant-design-vue';
import helpers from "../../../utils/helpers";
import IcPaperPlane from "../../Icon/IcPaperPlane.vue";
import {useChatListStore} from "../../../stores/chat_list";
import {BASE_URL, sendMessageMedia} from "../../../api";
import request from "../../../utils/request";
import {Vue3Lottie} from "vue3-lottie";

import loadingJson from '../../../assets/lottie/loading.json'
import IcLoading from "../../Icon/IcLoading.vue";

const fileList = ref<UploadProps['fileList']>([]);
const filePreview = ref([]);
const uploading = ref<boolean>(false);
const loading = ref<boolean>(false);
const imageUrl = ref<string>('');
const previewVisible = ref(false);
const previewImage = ref('');
const progressPercentage = ref(0);
const maxFileLength = ref(5);
const previewTitle = ref('');
const mediaType = ref('');
const isProcessing = ref(false);
const mediaUrl = ref(null);
const messageForm = ref("")

const chat = useChatListStore()
const emit = defineEmits(['onClose'])

const handleProgress = (e) => {
}

const removeMedia = () => {
  fileList.value = []
  mediaUrl.value = ''
}


const onKeyupMessage = async (event) => {
  if (messageForm.value.length > 0) {
    // await sendMessage()
  }
}

const onKeydownMessage = (event) => {
  messageForm.value = messageForm.value + "\n"
}

function playVideo() {
  const idVideo = document.getElementById('idVideo') as HTMLFormElement
  const playVideo = document.getElementById('playVideo') as HTMLFormElement
  idVideo.play()
  idVideo.setAttribute("controls", "controls")
  playVideo.style.display = "none"
}

function onPaused() {
  const idVideo = document.getElementById('idVideo') as HTMLFormElement
  const playVideo = document.getElementById('playVideo') as HTMLFormElement
  idVideo.removeAttribute("controls")
  playVideo.style.display = "block"
}

const getBase64 = (file: File) => {
  return new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.readAsDataURL(file);
    reader.onload = () => resolve(reader.result);
    reader.onerror = error => reject(error);
  });
}

const progress: UploadProps['progress'] = {
  strokeColor: {
    '0%': '#108ee9',
    '100%': '#87d068',
  },
  strokeWidth: 3,
  format: percent => `${parseFloat(percent.toFixed(2))}%`,
  class: 'test',
};
const handlePreview = async (file: UploadProps['fileList'][number]) => {
  if (!file.url && !file.preview) {
    file.preview = (await getBase64(file.originFileObj)) as string;
  }
  previewImage.value = file.url || file.preview;
  previewVisible.value = true;
  previewTitle.value = file.name || file.url.substring(file.url.lastIndexOf('/') + 1);
};

function getBinaryFromFile(file) {
  return new Promise((resolve, reject) => {
    const reader = new FileReader();

    reader.addEventListener("load", () => resolve(reader.result));
    reader.addEventListener("error", err => reject(err));

    reader.readAsText(file);
  });
}

const sendMedia = (chat_id, data) => {
  return request({
    url: `${BASE_URL}/whatsapp/send-media/${chat_id}`,
    method: 'post',
    data: data,
    onUploadProgress: function (progressEvent) {
      progressPercentage.value = parseInt(String(Math.round((progressEvent.loaded / progressEvent.total) * 100)));
    }.bind(progressPercentage.value)
  });
}

const removeFileList = (index) => {
  fileList.value.splice(index, 1);
  filePreview.value.splice(index, 1);
  mediaUrl.value = filePreview.value[0].media_url
}

const changePreview = (index) => {
  mediaUrl.value = filePreview.value[index].media_url
}

const onUpload = async () => {
  const formData = new FormData();
  formData.append('to', chat.detail.customer.phone);
  formData.append('type', mediaType.value);

  uploading.value = true;
  if (fileList.value.length > 1) {

    fileList.value.forEach((file: UploadProps['fileList'][number]) => {
      formData.append('file[]', file as any);
      formData.append('caption[]', helpers.encodeMessage(messageForm.value));
    });
    await helpers.Upload(`${BASE_URL}/whatsapp/send-media-multiple/${chat.selectedChat}`, formData, (percent) => {
          progressPercentage.value = percent
        }, null,
        (res) => {
          // ...
          if (res.data.code == 200) {
            // setTimeout(() => {
            message.success("Media has been uploaded & send to customer")
            // res.data.message = atob(res.data.message).toString()
            // if (res.data.chat) {
            //     res.data.chat.map((item) => {
            //         chat.messages.push(item)
            //         chat.sendMessageCount++
            //     })
            // }
            chat.fetchChatByCustomer(chat.selectedChat, 1).then((resp) => {
              console.log("testasnisandi")
              const messageArr = []
              chat.messages.map((msg) => {
                msg['message'] = helpers.whatsappFormatting(helpers.urlify(helpers.decodeMessage(msg.message)))
                msg['send_loading'] = false
                messageArr.push(msg)
              })
              helpers.scrollToBottomChat()
            }).catch((err) => {
            })
            fileList.value = []
            imageUrl.value = ''
            mediaUrl.value = ''
            messageForm.value = ''
            progressPercentage.value = 0
            emit("onClose", true)
            // }, 1000)
          }
          isProcessing.value = false
        }, (err) => {
          isProcessing.value = false
          progressPercentage.value = 0
          message.error(err.response.data.message)
        })
  } else {
    if (chat.replyChat.isReplying) {
      formData.append("reply_to_chat_id", chat.replyChat.chatId);
    }

    fileList.value.forEach((file: UploadProps['fileList'][number]) => {
      formData.append('file', file as any);
      formData.append('caption', helpers.encodeMessage(messageForm.value));
    });
    await helpers.Upload(`${BASE_URL}/whatsapp/send-media/${chat.selectedChat}`, formData, (percent) => {
          progressPercentage.value = percent
        }, null,
        (res) => {
          // ...
          if (res.data.code == 200) {
            // setTimeout(() => {
            message.success("Media has been uploaded & send to customer")
            // res.data.message = atob(res.data.message).toString()
            // if (res.data.chat) {
            //     res.data.chat.map((item) => {
            //         chat.messages.push(item)
            //         chat.sendMessageCount++
            //     })
            // }
            chat.fetchChatByCustomer(chat.selectedChat, 1).then((resp) => {
              console.log("halo")
              const messageArr = []
              chat.messages.map((msg) => {
                msg['message'] = helpers.whatsappFormatting(helpers.urlify(helpers.decodeMessage(msg.message)))
                msg['send_loading'] = false
                messageArr.push(msg)
              })
              helpers.scrollToBottomChat()
            }).catch((err) => {
            })
            fileList.value = []
            imageUrl.value = ''
            messageForm.value = ''
            mediaUrl.value = ''
            progressPercentage.value = 0
            emit("onClose", true)
            // }, 1000)
          }
          isProcessing.value = false
        }, (err) => {
          isProcessing.value = false
          progressPercentage.value = 0
          message.error(err.response.data.message)
        })
  }

}

const handleChange = (info: UploadChangeParam) => {
  const status = info.file.status;
  const regx = /^(image|video)\/.*$/
  const isJpgOrPng = regx.test(info.file.type);
  // message.info(`Wadaw ${file.type} == ${isJpgOrPng}`);

  if (!isJpgOrPng) {
    fileList.value = [];
    message.error('You can only upload Image or Video file!');
    return false
  }

  const regxImage = /^(image)\/.*$/
  const regxVideo = /^(video)\/.*$/
  const maxSize = {
    video: 10,
    image: 5
  }
  let limitUpload = 0
  if (regxImage.test(info.file.type)) {
    limitUpload = maxSize.image
    mediaType.value = "image"
  }
  if (regxVideo.test(info.file.type)) {
    limitUpload = maxSize.video
    mediaType.value = "video"
  }
  if (status !== 'uploading') {
    if (regxImage.test(info.file.type) || regxVideo.test(info.file.type)) {
      const isLt2M = info.file.size / 1024 / 1024 < limitUpload;
      if (!isLt2M) {
        message.error(`Image/Video must smaller than ${limitUpload}MB! `);
        fileList.value = [];
        return false
      }

      // File list cuma 1
      // fileList.value = [info.fileList[info.fileList.length - 1].originFileObj];
      let listFile = info.fileList


      if (listFile.length <= 5) {

        let fileArr = []
        let filePreviewArr = []
        listFile.map((item, index) => {
          fileArr.push(item.originFileObj)
          const obj = {}
          obj['media_url'] = URL.createObjectURL(item.originFileObj)
          obj['key'] = index
          filePreviewArr.push(obj)
        })
        fileList.value = fileArr;
        filePreview.value = filePreviewArr;

        const file = fileList.value
        mediaUrl.value = URL.createObjectURL(info.fileList[0].originFileObj)
        // chat.mediaURL = URL.createObjectURL(info.fileList[info.fileList.length - 1].originFileObj)
        // chat.setMediaUrl(URL.createObjectURL(info.fileList[info.fileList.length - 1].originFileObj))
        // helpers.getBase64(info.fileList[info.fileList.length - 1].originFileObj, (base64Url: string) => {
        //   imageUrl.value = base64Url;
        //   loading.value = false;
        // });
      }

    }
  }
  if (status === 'done') {
    message.success(`${info.file.name} file uploaded successfully.`);
  } else if (status === 'error') {
    message.error(`${info.file.name} file upload failed.`);
  }
};
const handleDrop = (e: DragEvent) => {
}
const beforeUpload: UploadProps['beforeUpload'] = file => {
  const regx = /^(image|video)\/.*$/
  const isJpgOrPng = regx.test(file.type);
  // message.info(`Wadaw ${file.type} == ${isJpgOrPng}`);

  if (!isJpgOrPng) {
    fileList.value = [];
    message.error('You can only upload Image or Video file!');
    return false
  }

  const regxImage = /^(image)\/.*$/
  const regxVideo = /^(video)\/.*$/
  const maxSize = {
    video: 10,
    image: 5
  }

  let limitUpload = 0
  if (regxImage.test(file.type)) {
    limitUpload = maxSize.image
    mediaType.value = "image"
  }
  if (regxVideo.test(file.type)) {
    limitUpload = maxSize.video
    mediaType.value = "video"
  }

  if (regxImage.test(file.type) || regxVideo.test(file.type)) {
    const isLt2M = file.size / 1024 / 1024 < limitUpload;
    if (!isLt2M) {
      message.error(`Image/Video must smaller than ${limitUpload}MB! `);
      fileList.value = [];
      return false
    }
    fileList.value = [file];
    chat.mediaURL = URL.createObjectURL(file)
    mediaUrl.value = URL.createObjectURL(file)
    // chat.setMediaUrl(URL.createObjectURL(file))
    // helpers.getBase64(fileList.value[0].originFileObj, (base64Url: string) => {
    //   imageUrl.value = base64Url;
    //   loading.value = false;
    // });
    return false;
  }
  return false;
};

watch(() => progressPercentage.value, (val) => {
  if (val > 0 && val < 100) {
    progressPercentage.value = val
  }

  if (val === 100) {
    isProcessing.value = true
  }
  // if (val >= 100) {
  //   setTimeout(() => {
  //     progressPercentage.value = 0
  //   }, 1000)
  // }
})
watch(() => mediaUrl.value, (val, oldValue) => {
  // if (val >= 100) {
  //   setTimeout(() => {
  //     progressPercentage.value = 0
  //   }, 1000)
  // }
})

watch(() => chat.onDropFiles, (val) => {
  const regxImage = /^(image)\/.*$/
  const regxVideo = /^(video)\/.*$/
  chat.showOnDropModal = false
  if (val.length > 0) {
    if (regxImage.test(val[0].type) || regxVideo.test(val[0].type)) {
      let fileArr = []
      let filePreviewArr = []
      val.map((item, index) => {
          // fileArr.push(item.originFileObj)
          const obj = {}
          // obj['media_url'] = URL.createObjectURL(item.originFileObj)
          obj['key'] = index
          filePreviewArr.push(obj)
        // fileList.value = fileArr;
        // filePreview.value = filePreviewArr;

        const file = fileList.value
        // mediaUrl.value = URL.createObjectURL( fileArr[0].originFileObj)
      })
    }
  }
})

watch(() => [...fileList.value], (val, oldValue) => {
  // if (val.length >= maxFileLength.value) {
  //   message.info('Hanya dapat memilih maximum 5 File!');
  // }
  // if (val >= 100) {
  //   setTimeout(() => {
  //     progressPercentage.value = 0
  //   }, 1000)
  // }
})
</script>

<style lang="css">
.ant-upload-select {
  width: 100%;
  height: 100%;
  position: relative;
  display: block;
}
</style>